/* hashtable.h -- Universal Hash Table with Linear Probing
 * 
 * Copyright (C) 2004 H Zero Seven
 *
 * Authors: Christian Volmering <risk@h07.org>
 * 
 * This file is part of the aexpl-ng distribution.
 * 
 * The aexpl-ng distribution is free software; you can redistribute it 
 * and/or modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either version 2 of 
 * the License, or (at your option) any later version.
 *
 * The aexpl-ng distribution is distributed in the hope that it will
 * be useful, but WITHOUT ANY WARRANTY; without even the implied 
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with the aexpl-ng distribution; if not, write to the 
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
 * MA 02111-1307 USA
 */

#ifndef _H07_HASHTABLE_H
#define _H07_HASHTABLE_H

#define OPT_NONE     0
#define OPT_STORE    1
#define OPT_OPTIMIZE 2

struct _hfunc_t
{
  unsigned int (*hash) (const void *, int);
  int (*equal) (const void *, const void *);
  void *(*copy) (const void *);
  void (*dispose) (const void *);
};

typedef struct _hfunc_t hfunc_t;
typedef struct _htable_t *htable_t;

extern hfunc_t KEY_STR;
extern hfunc_t KEY_LONG;

extern htable_t new_htable (int, int, hfunc_t);
extern void free_htable (htable_t);
extern int htable_insert (htable_t, const void *, const void *);
extern int htable_delete (htable_t, const void *);
extern const void *htable_lookup (htable_t, const void *);
extern int htable_contains (htable_t, const void *);
extern int htable_size (htable_t);
extern int htable_entries (htable_t);
extern int htable_clear (htable_t);
extern int htable_resize (htable_t, int);

#endif /* _H07_HASHTABLE_H */

/*
 * Local variables:
 * tab-width: 8
 * c-basic-offset: 8
 * End:
 * vim600: noet nonu sw=8 ts=8 fdm=marker
 * vim<600: noet nonu sw=8 ts=8
 */
