/* alert.c -- Alert handling
 * 
 * Copyright (C) 2004 H Zero Seven
 *
 * Authors: Christian Volmering <risk@h07.org>
 * 
 * This file is part of the aexpl-ng distribution.
 * 
 * The aexpl-ng distribution is free software; you can redistribute it 
 * and/or modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either version 2 of 
 * the License, or (at your option) any later version.
 *
 * The aexpl-ng distribution is distributed in the hope that it will
 * be useful, but WITHOUT ANY WARRANTY; without even the implied 
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with the aexpl-ng distribution; if not, write to the 
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
 * MA 02111-1307 USA
 */

#include "config.h"
#include "sendmail.h"

#include <stdlib.h>
#include <stdio.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <unistd.h>
#include <syslog.h>
#include <log.h>
#include <string.h>
#include <grp.h>
#include <pwd.h>

#ifdef DMALLOC
#  include <dmalloc.h>
#endif

/* {{{ alert */
int
alert (char *file, char *mail_adr, char *cmd, int uid, int gid)
{
  char content[512];
  pid_t pid;
  int status;
  struct passwd *passwd = getpwuid (uid);
  struct group *group = getgrgid (gid);

  write_log (LOG_ALERT, "Found suspious file: "
	     "%s uid(%i) gid(%i)", file, uid, gid);

  snprintf (content, 512, "AntiExploit Alert!\n\n"
	    "File:  %s\nUser:  %i (%s)\nGroup: %i (%s)\n",
	    file,
	    uid, passwd ? passwd->pw_name : "unknown",
	    gid, group ? group->gr_name : "unknown");

  if (!sendmail (mail_adr, content))
    write_log (LOG_ERR, "Error sending mail: %s", strerror (errno));

  if (!cmd)
    return (1);

  if ((pid = fork ()) == -1)
    {
      write_log (LOG_ERR, "Couldn't fork: %s", strerror (errno));

      return (0);
    }
  else if (pid == 0)
    {
      char __uid[11], __gid[11];
      char *argv[5];

      snprintf (__uid, 11, "%i", (unsigned int) uid);
      snprintf (__gid, 11, "%i", (unsigned int) gid);

      argv[0] = cmd;
      argv[1] = file;
      argv[2] = __uid;
      argv[3] = __gid;
      argv[4] = NULL;

      execv (cmd, argv);

      syslog (LOG_ERR, "Couldn't execute process: %s", strerror (errno));
      exit (1);
    }

  while ((waitpid (pid, &status, 0) == -1) && (errno == EINTR));

  if (WIFEXITED (status))
    return (WEXITSTATUS (status));
  else
    return (0);
}

/* }}} */

/*
 * Local variables:
 * tab-width: 8
 * c-basic-offset: 8
 * End:
 * vim600: noet nonu sw=8 ts=8 fdm=marker
 * vim<600: noet nonu sw=8 ts=8
 */
