/* loaddb.c -- Howto load the exploid database
 *
 * Copyright (C) 2004 H Zero Seven
 *
 * Authors: Benjamin Schweizer <gopher@h07.org>
 *          Enrico Kern <phantom@h07.org>
 *          Christian Volmering <risk@h07.org>
 *
 * This file is part of the aexpl-ng distribution.
 *
 * The aexpl-ng distribution is free software; you can redistribute it
 * and/or modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * The aexpl-ng distribution is distributed in the hope that it will
 * be useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with the aexpl-ng distribution; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include "config.h"
#include "loaddb.h"
#include "hashtable.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <limits.h>

#ifdef DMALLOC
#  include <dmalloc.h>
#endif

#define MAX_LINE_LEN (PATH_MAX + 34)

/* {{{ loaddb */
int
loaddb (htable_t * db, const char *filename)
{
  FILE *fp;
  int lines = 0, len = 0;
  char line[MAX_LINE_LEN];
  char *str;
  char fname[PATH_MAX];
  char md5sum[33];

  if ((fp = fopen (filename, "r")))
    {
      if (!(*db = new_htable (4000, OPT_STORE, KEY_STR)))
	return (-1);

      while (fgets (line, MAX_LINE_LEN, fp) > 0)
	{
	  str = line;

	  strncpy (md5sum, str, 32);
	  md5sum[32] = '\0';

	  str += 33;
	  len = strlen (str) - 1;
	  strncpy (fname, str, len);
	  fname[len] = '\0';

	  htable_insert (*db, (void *) md5sum, (void *) fname);

	  lines++;
	}
      fclose (fp);

      return lines;
    }

  return (-1);
}

/* }}} */

/*
 * Local variables:
 * tab-width: 8
 * c-basic-offset: 8
 * End:
 * vim600: noet nonu sw=8 ts=8 fdm=marker
 * vim<600: noet nonu sw=8 ts=8
 */
