/* main.c -- AntiExploit
 * 
 * Copyright (C) 2004 H Zero Seven
 *
 * Authors: Enrico Kern <phantom@h07.org>
 *          Christian Volmering <risk@h07.org>
 * 
 * This file is part of the aexpl-ng distribution.
 * 
 * The aexpl-ng distribution is free software; you can redistribute it 
 * and/or modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either version 2 of 
 * the License, or (at your option) any later version.
 *
 * The aexpl-ng distribution is distributed in the hope that it will
 * be useful, but WITHOUT ANY WARRANTY; without even the implied 
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with the aexpl-ng distribution; if not, write to the 
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
 * MA 02111-1307 USA
 */

#include "aeconfig.h"
#include "loaddb.h"
#include "dazukoio.h"
#include "update.h"
#include "queue.h"
#include "hashtable.h"
#include "md5sum.h"
#include "sendmail.h"
#include "alert.h"
#include "daemon.h"
#include "proctitle.h"
#include "lock.h"
#include "config.h"
#include "fqueue.h"
#include "log.h"

#include <stdio.h>
#include <signal.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <limits.h>
#include <sys/wait.h>
#include <syslog.h>

#ifdef DMALLOC
#  include <dmalloc.h>
#endif

#if !defined CLD_EXITED
#  define CLD_EXITED 1 // Child has exited.
#endif
#if !defined CLD_KILLED
#  define CLD_KILLED 2 // Child was killed.
#endif
#if !defined CLD_DUMPED
#  define CLD_DUMPED 3 // Child terminated abnormally.
#endif

static unsigned short running = 1;
static htable_t db;
static fqueue_t filequeue;
static pid_t worker_th = -1;
static pid_t main_th = -1;

void sigterm_handler (int sig);
void sighup_handler (int sig);
void sigchld_handler (int sig, siginfo_t * info, void *context);
int register_dazuko (void);
void worker_loop (void);
void main_loop (void);
int configure (const char *config);
int initialise (void);
void exit_cleanup (void);
void opt_version (void);
void opt_usage (const char *self);
void opt_update (const char *config);
void opt_start (int argc, char **argv);

/* {{{ Configuration section */

// Path to exploit database
static char exploit_db[PATH_MAX];
// Uri where the exploit database is downloaded from
static char update_url[PATH_MAX];
// Mail adress for exploit alerts
static char mail_addr[128];
// User script executed on alterts
static char user_script[PATH_MAX];
// Lock file for aexpl (stores pid)
static char lock_file[PATH_MAX];
// Path to the shared library config file
static char ld_so_conf[PATH_MAX];
// Log file for aexpl
static char log_file[PATH_MAX];
// Whether to run the user script or not
static short run_script;
// Size of inode table
static int inot_size;
// Maximum file size to check
static int max_file_size;
// Size of the file queue
static int queue_size;
// Log facility for syslog
static int facility;

static cfg_element_t ae_config[] = {
  {"/aexpl/analyze/exploit_db", exploit_db, etype_str, PATH_MAX},
  {"/aexpl/analyze/update_url", update_url, etype_str, PATH_MAX},
  {"/aexpl/reports/mail_addr", mail_addr, etype_str, 128},
  {"/aexpl/reports/log_file", log_file, etype_str, PATH_MAX},
  {"/aexpl/scripts/user_script", user_script, etype_str, PATH_MAX},
  {"/aexpl/core/lock_file", lock_file, etype_str, PATH_MAX},
  {"/aexpl/core/ld_so_conf", ld_so_conf, etype_str, PATH_MAX},
  {"/aexpl/scripts/run_script", &run_script, etype_bool},
  {"/aexpl/core/inot_size", &inot_size, etype_int},
  {"/aexpl/core/max_file_size", &max_file_size, etype_int},
  {"/aexpl/core/queue_size", &queue_size, etype_int},
  {"/aexpl/reports/facility", &facility, etype_int},
  {0, 0, 0, 0}
};

/* }}} */

/* {{{ main */
int
main (int argc, char **argv)
{
  static char optstring[] = "c:vhu:";

  if (argc < 2)
    opt_usage (*argv);

  switch (getopt (argc, argv, optstring))
    {
    case 'v':
      opt_version ();
      break;

    case 'h':
      opt_usage (*argv);
      break;

    case 'u':
      opt_update (argv[2]);
      break;

    case 'c':
      opt_start (argc, argv);
      break;

    default:
      opt_usage (*argv);
    }

  return (0);
}

/* }}} */

/* {{{ opt_version */
void
opt_version (void)
{
  printf ("AntiExploit %s\n", VERSION);
  printf ("Copyright (C) 2004 H Zero Seven\n\n");

  printf ("This program comes with ABSOLUTELY NO WARRANTY.\n");
  printf ("This is free software, and you are welcome to "
	  "redistribute it\nunder certain conditions. "
	  "See the file COPYING for details.\n");

  exit (0);
}

/* }}} */

/* {{{ opt_usage */
void
opt_usage (const char *self)
{
  printf ("Usage: %s [options]\nOptions:\n", self);
  printf ("  %-15s %s\n", "-h", "Print this message and exit.");
  printf ("  %-15s %s\n", "-v", "Show version information.");
  printf ("  %-15s %s\n", "-c <config>", "Start AntiExploit.");
  printf ("  %-15s %s\n", "-u <config>", "Update the Exploit database.");
  printf ("\nReport bugs to %s.\n", "phantom@h07.org or risk@h07.org");

  exit (0);
}

/* }}} */

/* {{{ opt_start */
void
opt_start (int argc, char **argv)
{
  struct sigaction sa;

  if (getuid () != 0)
    {
      printf ("Only root can run AntiExploit!\n");

      exit (1);
    }

  atexit (exit_cleanup);

  if (!configure (argv[2]))
    exit (1);

  printf ("Detaching, please check the log file to"
	  " see if everything is ok!\n");

  if (!detach (1, 0))
    exit (1);

  if (!initialise ())
    exit (1);

  if (read_pid_file (lock_file))
    {
      write_log (LOG_INFO,
		 "Another instance of AntiExploit is already running!");

      exit (1);
    }

  // create file queue
  if (!(filequeue = new_fqueue (queue_size)))
    {
      write_log (LOG_ERR, "Error creating file queue: %s", strerror (errno));

      exit (1);
    }

  if (!create_pid_file (lock_file))
    {
      write_log (LOG_ERR, "Error creating lock file: %s", strerror (errno));

      exit (1);
    }

  write_log (LOG_INFO, "AntiExploit started.");

  set_proc_title (argc, argv, "[aexpl]");

  sigemptyset (&sa.sa_mask);
  sa.sa_flags = SA_RESTART;

  sa.sa_handler = sigterm_handler;
  if (sigaction (SIGTERM, &sa, NULL) == -1)
    {
      write_log (LOG_ERR, "Error setting up signal handler: %s",
		 strerror (errno));

      exit (1);
    }

  sa.sa_handler = sighup_handler;
  if (sigaction (SIGHUP, &sa, NULL) == -1)
    {
      write_log (LOG_ERR, "Error setting up signal handler: %s",
		 strerror (errno));

      exit (1);
    }

  sa.sa_handler = NULL;
  sa.sa_sigaction = sigchld_handler;
  sa.sa_flags = SA_RESTART | SA_SIGINFO;
  if (sigaction (SIGCHLD, &sa, NULL) == -1)
    {
      write_log (LOG_ERR, "Error setting up signal handler: %s",
		 strerror (errno));

      exit (1);
    }

  main_th = getpid ();
  switch (worker_th = fork ())
    {
    case 0:
      worker_loop ();
      break;

    case -1:
      write_log (LOG_ERR, "Couldn't fork: %s", strerror (errno));
      break;

    default:
      main_loop ();
    }

  remove_pid_file (lock_file);

  exit (0);
}

/* }}} */

/* {{{ opt_update */
void
opt_update (const char *config)
{
  int ae_pid;

  if (!configure (config))
    exit (1);

  if (!update (update_url, exploit_db))
    {
      printf ("Update failed!\n");

      exit (1);
    }
  else
    printf ("Update complete.\n");

  if ((ae_pid = read_pid_file (lock_file)))
    {
      printf ("Sending SIGHUP to active AntiExploit process: ");

      if (kill (ae_pid, SIGHUP) != 0)
	{
	  perror ("Couldn't send signal");

	  exit (1);
	}
      else
	printf ("done.\n");
    }

  exit (0);
}

/* }}} */

/* {{{ sigterm_handler */
void
sigterm_handler (int sig)
{
  int chld_status;

  write_log (LOG_INFO, "Process %ld terminating ...", (long) getpid ());

  running = 0;

  if (getpid () == main_th)
    {
      if (dazukoUnregister () != 0)
	write_log (LOG_ERR, "Failed to unregister with Dazuko.");

      free_fqueue (filequeue);

      if (worker_th != -1)
	{
	  // Kill the worker thread and wait until it is terminated
	  kill (worker_th, SIGTERM);

	  while ((waitpid (worker_th, &chld_status, 0) == -1)
		 && (errno == EINTR));
	}
    }
}

/* }}} */

/* {{{ sighup_handler */
void
sighup_handler (int sig)
{
  int num_sums;
  pid_t pid;

  pid = getpid ();

  if (pid == main_th)
    {
      if (worker_th != -1)
	{
	  write_log (LOG_INFO, "Sending SIGHUP to child: %ld",
		     (long) worker_th);

	  kill (worker_th, SIGHUP);
	}
    }
  else
    {
      write_log (LOG_INFO, "Got SIGHUP, updating database.");

      free_htable (db);
      if ((num_sums = loaddb (&db, exploit_db)) < 0)
	{
	  write_log (LOG_ERR, "Error reading database.");
	  running = 0;
	}

      write_log (LOG_INFO, "Done, now checking for %i exploids.", num_sums);
    }
}

/* }}} */

/* {{{ sigchld_handler */
void
sigchld_handler (int sig, siginfo_t * info, void *context)
{
  if (getpid () != main_th)
    return;

  switch (info->si_code)
    {
    case CLD_EXITED:
      kill (main_th, SIGTERM);

      break;

    case CLD_KILLED:
      if (info->si_status)
	write_log (LOG_INFO, "Process %d terminated due to signal: %d",
		   info->si_pid, info->si_status);
      kill (main_th, SIGTERM);

      break;

    case CLD_DUMPED:
      write_log (LOG_INFO, "Process %d terminated abnormally.", info->si_pid);
      kill (main_th, SIGTERM);

      break;
    }
}

/* }}} */

/* {{{ register_dazuko */
int
register_dazuko (void)
{
  if (dazukoRegister ("ANTI_EXPLOIT", "r") != 0)
    return (0);
  else
    {
      FILE *ldfp;
      char ldline[80];
      char *ldstr;

      // Add all library paths to exclude
      if ((ldfp = fopen (ld_so_conf, "r")))
	{
	  while (fgets (ldline, 80, ldfp) > 0)
	    {
	      ldstr = ldline;
	      dazukoAddExcludePath (ldstr);
	    }
	}

      if (dazukoAddIncludePath ("/") != 0 ||
	  dazukoAddExcludePath ("/dev") != 0 ||
	  dazukoAddExcludePath ("/proc") != 0 ||
	  dazukoAddExcludePath ("/lib") != 0)
	{
	  write_log (LOG_ERR, "Could not add include or exclude paths.");

	  return (0);
	}

      // set access mask
      if (dazukoSetAccessMask (DAZUKO_ON_OPEN |
			       DAZUKO_ON_CLOSE |
			       DAZUKO_ON_CLOSE_MODIFIED |
			       DAZUKO_ON_EXEC) != 0)
	{
	  write_log (LOG_ERR, "Failed to set access mask.");
	  dazukoUnregister ();

	  return (0);
	}

      return (1);
    }
}

/* }}} */

/* {{{ worker_loop */
void
worker_loop (void)
{
  char file[PATH_MAX];
  long *ino;
  char sum[33];
  struct stat file_stat;
  htable_t ino_table;
  queue_t ino_queue;

  void on_dequeue (const void *ptr, int action)
  {
    long *ino_ptr = (long *) ptr;

    // When action is dequeue the queue removed the last inode
    // because it's filled more than max_size, so we alsow delete
    // the inode from the hash table.
    if (action == NOTIFY_DEQUEUE)
      {
	htable_delete (ino_table, ino_ptr);
	free (ino_ptr);
      }
  }

  write_log (LOG_INFO, "Worker thread woken up.");

  // Try to create the inode table to store allready scanned files.
  if (!(ino_table = new_htable (inot_size, 0, KEY_LONG)))
    {
      write_log (LOG_ERR, "Error creating inotab: %s", strerror (errno));

      exit (1);
    }

  if (!(ino_queue = new_queue (inot_size)))
    {
      write_log (LOG_ERR, "Error creating inoqueue: %s", strerror (errno));

      exit (1);
    }

  notify (ino_queue, &on_dequeue);

  open_fqueue (filequeue);

  while (running)
    {
      if (!fdequeue (filequeue, file))
	continue;

      if (stat (file, &file_stat) == -1)
	continue;

      if (!(ino = (long *) malloc (sizeof (long))))
	{
	  write_log (LOG_ERR, "Couldn't allocate inode: %s",
		     strerror (errno));

	  exit (1);
	}

      // A inode is not unique, so we do a bitwise xor with the device id.
      *ino = file_stat.st_dev ^ file_stat.st_ino;

      // If the table contains the inode, skip the checks.
      if (!htable_contains (ino_table, ino))
	{
	  if (!md5sum_from_file (file, sum))
	    free (ino);
	  else
	    {
	      if (htable_contains (db, sum))
		alert (file, mail_addr,
		       run_script ? user_script : NULL,
		       file_stat.st_uid, file_stat.st_gid);

	      // Insert inode in the hashtable tp prevend multible checks. 
	      htable_insert (ino_table, (void *) ino, NULL);

	      // Alsow insert into ino queue.
	      //
	      // The queue will remove the last inode if it's filled more
	      // than max_size and the notify callback will than remove
	      // the inode from the hash table.
	      enqueue (ino_queue, (void *) ino);
	    }
	}
      else
	free (ino);
    }

  free_queue (ino_queue);
  free_htable (ino_table);

  exit (0);
}

/* }}} */

/* {{{ main_loop */
void
main_loop (void)
{
  struct dazuko_access *acc = NULL;
  int fail_count = 0;

  while (running)
    {
      if (acc != NULL)
	dazukoReturnAccess (&acc);

      // Wait for a access event.
      if (dazukoGetAccess (&acc) == 0)
	{
	  fail_count = 0;

	  if (!acc->set_event)
	    continue;

	  // Exclude files bigger than MAX_FILE_SIZE.
	  if (acc->set_file_size && (acc->file_size > max_file_size))
	    continue;

	  switch (acc->event)
	    {
	    case DAZUKO_ON_OPEN:
	      fenqueue (filequeue, acc->filename);
	      break;

	    case DAZUKO_ON_CLOSE:
	      fenqueue (filequeue, acc->filename);
	      break;

	    case DAZUKO_ON_CLOSE_MODIFIED:
	      fenqueue (filequeue, acc->filename);
	      break;

	    case DAZUKO_ON_EXEC:
	      fenqueue (filequeue, acc->filename);
	      break;
	    }
	}
      else
	{
	  fail_count++;

	  if (fail_count > 2)
	    {
	      write_log (LOG_ERR, "Error getting access three times,"
			 " terminating.");

	      running = 0;
	    }
	}
    }
}

/* }}} */

/* {{{ configure */
int
configure (const char *config)
{
  char err_buf[128];

  if (!cfg_setup (config))
    {
      printf ("Configuration error: %s\n", "Syntax error.");

      return (0);
    }

  if (!cfg_read_elements (ae_config, err_buf, 128))
    {
      printf ("Configuration error: %s\n", err_buf);

      return (0);
    }

  return (1);
}

/* }}} */

/* {{{ initialise */
int
initialise (void)
{

  unsigned short result = 1;
  int entries = 0;

  setup_log ("AntiExploit", (facility != -1) ? facility : LOG_DAEMON,
	     log_file);

  write_log (LOG_INFO, "Starting initialization.");

  // Load the exploit database.
  if ((entries = loaddb (&db, exploit_db)) < 0)
    {
      write_log (LOG_ERR,
		 "Could not load exploit-database, please verify "
		 "that the file you specified in aexpl.conf "
		 "exist, otherwhise check the README!");

      result = 0;
    }
  else
    {
      write_log (LOG_INFO, "Exploit database loaded with %i entries.",
		 entries);

      // If exploit database loaded successfully, register with dazuko.
      if (register_dazuko ())
	{
	  write_log (LOG_INFO, "Successfully registered with dazuko.");

	  result = 1;
	}
      else
	{
	  write_log (LOG_ERR,
		     "Could not register with dazuko please "
		     "verify that dazuko is installed and/or "
		     "read the README of aexpl");

	  result = 0;
	}
    }

  if (result)
    write_log (LOG_INFO, "Initialization complete.");
  else
    write_log (LOG_INFO, "Initialization failed!");

  return (result);
}

/* }}} */

/* {{{ exit_cleanup */
void
exit_cleanup (void)
{
  teardown_log ();
  dazukoUnregister ();
  cfg_teardown ();
}

/* }}} */

/*
 * Local variables:
 * tab-width: 8
 * c-basic-offset: 8
 * End:
 * vim600: noet nonu sw=8 ts=8 fdm=marker
 * vim<600: noet nonu sw=8 ts=8
 */
