/* sendmail.c -- Send mail via mailutil
 * 
 * Copyright (C) 2004 H Zero Seven
 *
 * Authors: Enrico Kern <phantom@h07.org>
 *          Christian Volmering <risk@h07.org>
 * 
 * This file is part of the aexpl-ng distribution.
 * 
 * The aexpl-ng distribution is free software; you can redistribute it 
 * and/or modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either version 2 of 
 * the License, or (at your option) any later version.
 *
 * The aexpl-ng distribution is distributed in the hope that it will
 * be useful, but WITHOUT ANY WARRANTY; without even the implied 
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with the aexpl-ng distribution; if not, write to the 
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
 * MA 02111-1307 USA
 */

#include "config.h"

#include <unistd.h>
#include <sys/wait.h>
#include <stdio.h>
#include <sys/types.h>
#include <fcntl.h>
#include <string.h>
#include <sys/utsname.h>
#include <sys/param.h>
#include <sys/time.h>

#ifdef DMALLOC
#  include <dmalloc.h>
#endif

#define MAIL_WRITE_TIMEOUT 5

/* {{{ sendmail */
int
sendmail (const char *to, const char *content)
{
  struct utsname name;
  struct timeval timeout;
  FILE *pipe;
  char mail_cmd[MAXPATHLEN];
  fd_set pipe_out;
  int pipe_no;

  // os release, hostname etc. 
  if (uname (&name) == -1)
    return (0);

  // pipe to mail etc.
  snprintf (mail_cmd, MAXPATHLEN,
	    "%s -s 'AntiExploit Report for %s' %s",
	    MAIL_PROG, name.nodename, to);

  if (!(pipe = popen (mail_cmd, "w")))
    return (0);

  // get file descriptor number
  pipe_no = fileno (pipe);

  // clear select set and add descriptor
  FD_ZERO (&pipe_out);
  FD_SET (pipe_no, &pipe_out);

  // setup timeout
  timeout.tv_sec = MAIL_WRITE_TIMEOUT;
  timeout.tv_usec = 0;

  // wait for stream to become writeable
  if (select (fileno (pipe) + 1, NULL, &pipe_out, NULL, &timeout) > 0)
    {
      // write content to sendmail 
      fprintf (pipe, "%s\n", content);
      fflush (pipe);
    }

  return (!pclose (pipe));
}

/* }}} */

/*
 * Local variables:
 * tab-width: 8
 * c-basic-offset: 8
 * End:
 * vim600: noet nonu sw=8 ts=8 fdm=marker
 * vim<600: noet nonu sw=8 ts=8
 */
