/* update.c -- Update the exploid database via curl
 * 
 * Copyright (C) 2004 H Zero Seven
 *
 * Authors: Enrico Kern <phantom@h07.org>
 *          Christian Volmering <risk@h07.org>
 * 
 * This file is part of the aexpl-ng distribution.
 * 
 * The aexpl-ng distribution is free software; you can redistribute it 
 * and/or modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either version 2 of 
 * the License, or (at your option) any later version.
 *
 * The aexpl-ng distribution is distributed in the hope that it will
 * be useful, but WITHOUT ANY WARRANTY; without even the implied 
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with the aexpl-ng distribution; if not, write to the 
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
 * MA 02111-1307 USA
 */

#include "config.h"

#include <stdio.h>
#include <curl/curl.h>
#include <curl/types.h>
#include <curl/easy.h>

#ifdef DMALLOC
#  include <dmalloc.h>
#endif

struct _web_file_t
{
  char *filename;
  FILE *stream;
};

/* {{{ _write_file */
static int
_write_file (void *buffer, size_t size, size_t nmemb, void *stream)
{
  struct _web_file_t *out = (struct _web_file_t *) stream;

  if (out && !out->stream)
    {
      // open file for writing
      out->stream = fopen (out->filename, "wb");

      if (!out->stream)
	return (-1);
    }

  return fwrite (buffer, size, nmemb, out->stream);
}

/* }}} */

/* {{{ update */
int
update (const char *url, char *storefile)
{
  CURL *curl;
  CURLcode res;
  struct curl_slist *headers = NULL;
  char user_agent[100];
  struct _web_file_t webfile = { storefile, NULL };

  curl_global_init (CURL_GLOBAL_DEFAULT);

  curl = curl_easy_init ();

  if (curl)
    {
      snprintf (user_agent, 100, "User-Agent: AntiExploit %s", VERSION);
      headers = curl_slist_append (headers, user_agent);

      curl_easy_setopt (curl, CURLOPT_HTTPHEADER, headers);
      curl_easy_setopt (curl, CURLOPT_URL, url);
      curl_easy_setopt (curl, CURLOPT_WRITEFUNCTION, _write_file);

      // Set a pointer to our struct to pass to the callback
      curl_easy_setopt (curl, CURLOPT_WRITEDATA, &webfile);

      res = curl_easy_perform (curl);

      curl_slist_free_all (headers);
      curl_easy_cleanup (curl);

      if (CURLE_OK != res)
	return (0);
    }

  if (webfile.stream)
    fclose (webfile.stream);

  curl_global_cleanup ();

  return (1);
}

/* }}} */

/*
 * Local variables:
 * tab-width: 8
 * c-basic-offset: 8
 * End:
 * vim600: noet nonu sw=8 ts=8 fdm=marker
 * vim<600: noet nonu sw=8 ts=8
 */
